/*
 --------------------------------------------------------------------------
 ITU-T G.720.1 Source Code
 Software Release 1.1a (2010-10)
 (C) 2009 Huawei Technologies Co., LTD. 
 --------------------------------------------------------------------------
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "constdef_fx.h"
#include "typedef_fx.h"
#include "commandpars.h"
#include "wmops.h"

#ifndef GVAD

Word32 getcommandpars(Word16 argc, 
					Word8 *argv[], 
					Word16 *nb, 
					Word16 *frame_size, 
					Word16 *mode, 
					Word16 *op,
					FILE **f_speech, 
					FILE **f_output)
{
	Word32 fsize;
#ifdef DRAM
    accDRAM(FRAME_COUNT, 2);
#endif
    fprintf(stderr, "\n");
    fprintf(stderr, "==================================================\n");
    fprintf(stderr, " G.720.1 Generic Sound Activity Detector, 2010-01\n");
    fprintf(stderr, "==================================================\n");
    fprintf(stderr, "\n");

    /* open speech file and result file (output decision result) */
    if (argc!=6)
    {
        fprintf(stderr, "Usage : gsad rate_option mode_option op_option speech_file decision_file\n");
        fprintf(stderr, "\n");
        fprintf(stderr, "Format for speech_file:\n");
        fprintf(stderr, "  Speech is read from a binary, headerless 16-bit linear PCM file.\n");
        fprintf(stderr, "\n");
        fprintf(stderr, "Format for decision_file:\n");
        fprintf(stderr, "  Described in binary of 8 bits.\n");

        fprintf(stderr, "\n");
        fprintf(stderr, "rate_option: 8k  16k (2 sample rate options) \n");
        fprintf(stderr, " ========================================================\n");
        fprintf(stderr, " rate_option:    8k         16k\n");
        fprintf(stderr, " sampling rate:  8 kHz      16 kHz\n");
        fprintf(stderr, " ========================================================\n");
		fprintf(stderr, "\n");
        fprintf(stderr, "mode_option: LC  FC\n");
        fprintf(stderr, " ========================================================\n");
        fprintf(stderr, " LC: (Low complexity) VAD mode\n");
        fprintf(stderr, " FC: (Full complexity) GSAD mode\n");
        fprintf(stderr, " ========================================================\n");
        fprintf(stderr, "op_option: 0/1/2  (operating point) \n");
        fprintf(stderr, " ========================================================\n");
        fprintf(stderr, " option:   0   Balanced operating point\n");
        fprintf(stderr, " option:   1   Quality-preferred operating point\n");
		fprintf(stderr, " option:   2   Bandwidth-saving preferred operating point\n");
        fprintf(stderr, " ========================================================\n");
        fprintf(stderr, "\n");
        exit(0);
    }
    
    if (strcmp(argv[1], "8k")==0)
    {
        *nb = 1;
        *frame_size = SAMPLE8k;
		printf("Operating at 8 kHz sampling frequency\n");
    }
	else
	{
		if (strcmp(argv[1], "16k")==0)
		{
			*nb = 0;
			*frame_size = SAMPLE16k;
			printf("Operating at 16 kHz sampling frequency\n");
		}
		else
		{
			fprintf(stderr, "Error specifying sampling rate %s !!\n", argv[1]);
			exit(0);
		}
	}
	if (strcmp(argv[2], "LC")==0)
    {
		*mode = 1;
		printf("Operating in LC mode\n");
    }
	else
	{
		if (strcmp(argv[2], "FC")==0)
		{
			*mode = 0;
			printf("Operating in FC mode\n");
		}
		else
		{
			fprintf(stderr, "Error specifying mode: LC for (low complexity) VAD mode and FC for (full complexity) GSAD mode %s !!\n");
			exit(0);	
		}
	}
	if (atoi(argv[3])==0)
	{
		*op = 0;
		printf("Using BALANCED operating point\n");
	}
	else
	{
		if (atoi(argv[3])==1)
		{
			*op = 1;
			printf("Using QUALITY-PREFERRED operating point\n");
		}
		else
		{
			if (atoi(argv[3])==2)
			{
				*op = 2;
				printf("Using BANDWIDTH-SAVING operating point\n");
			}
			else
			{
				fprintf(stderr, "Error specifying operating point %s !!\n", argv[3]);
				exit(0);
			}
		}
	}
    argv++;
    argv++;
    argv++;     
    fsize = f_size_Word16(argv[1]);

    if ((*f_speech = fopen(argv[1], "rb"))==NULL)
    {
        fprintf(stderr, "Error opening input file  %s !!\n", argv[1]);
        exit(0);
    }
    fprintf(stderr, "Input speech file:  %s\n", argv[1]);
	argv++;

    if ((*f_output = fopen(argv[1], "wb"))==NULL)
    {
        fprintf(stderr, "Error opening output decision file %s !!\n", argv[1]);
        exit(0);
    }
    fprintf(stderr, "Output decision file:  %s\n", argv[1]);
    argv++;
#ifdef DRAM
    accDRAM(FRAME_COUNT, -2);
#endif
    return(fsize);
}

#else 
Word32 getcommandpars(Word16 argc, 
					  Word8 *argv[], 
					  Word16 *nb, 
					  Word16 *frame_size, 
					  Word16 *op,
					  FILE **f_speech, 
					  FILE **f_output)
{
	Word32 fsize;
#ifdef DRAM
    accDRAM(FRAME_COUNT, 2);
#endif
    fprintf(stderr, "\n");
    fprintf(stderr, "=======================================================================\n");
    fprintf(stderr, "            G.720.1 Generic Sound Activity Detector, 2010-01\n");
    fprintf(stderr, "=======================================================================\n");
    fprintf(stderr, "\n");

    /* Open speech file and result file (output decision result) */
	if (argc!=5)
    {
        fprintf(stderr, "Usage : gsad rate_option op_option speech_file decision_file\n");
        fprintf(stderr, "\n");
        fprintf(stderr, "Format for speech_file:\n");
        fprintf(stderr, "  Speech is read from a binary, headerless 16-bit linear PCM file.\n");
        fprintf(stderr, "\n");
        fprintf(stderr, "Format for decision_file:\n");
        fprintf(stderr, "  Described in binary of 8 bits.\n");

        fprintf(stderr, "\n");
        fprintf(stderr, "rate_option: 8k  16k (2 sample rate options) \n");
        fprintf(stderr, " ========================================================\n");
        fprintf(stderr, " rate_option:    8k         16k\n");
        fprintf(stderr, " sampling rate:  8 kHz      16 kHz\n");
        fprintf(stderr, " ========================================================\n");
        fprintf(stderr, "\n");
        fprintf(stderr, "op_option: 0/1/2  (operating point) \n");
        fprintf(stderr, " ========================================================\n");
        fprintf(stderr, " option:   0   Balanced operating point\n");
        fprintf(stderr, " option:   1   Quality-preferred operating point\n");
        fprintf(stderr, " option:   2   Bandwidth-saving preferred operating point\n");
        fprintf(stderr, " ========================================================\n");
        fprintf(stderr, "\n");
        exit(0);
    }
    
    if (strcmp(argv[1], "8k")==0)
    {
        *nb = 1;
        *frame_size = SAMPLE8k;
    }
	else
	{
		if (strcmp(argv[1], "16k")==0)
		{
			*nb = 0;
			*frame_size = SAMPLE16k;
		}
		else
		{
			fprintf(stderr, "Error specifying sampling rate %s !!\n", argv[1]);
			exit(0);
		}
	}
	if (atoi(argv[2])==0)
	{
		*op = 0;
	}
	else
	{
		if (atoi(argv[2])==1)
		{
			*op = 1;
		}
		else
		{
			if (atoi(argv[2])==2)
			{
				*op = 2;
			}
			else
			{
				fprintf(stderr, "Error specifying operating point %s !!\n", argv[2]);
				exit(0);
			}
		}
	}		
    argv++;
    argv++;
    fsize = f_size_Word16(argv[1]);

    if ((*f_speech = fopen(argv[1], "rb"))==NULL)
    {
        fprintf(stderr, "Error opening input file  %s !!\n", argv[1]);
        exit(0);
    }
    fprintf(stderr, "Input speech file:  %s\n", argv[1]);
	argv++;

    if ((*f_output = fopen(argv[1], "wb"))==NULL)
    {
        fprintf(stderr, "Error opening output decision file %s !!\n", argv[1]);
        exit(0);
    }
    fprintf(stderr, "Output decision file:  %s\n", argv[1]);
    argv++;
#ifdef DRAM
    accDRAM(FRAME_COUNT, -2);
#endif
    return(fsize);
}
#endif

Word32 f_size_Word16(Word8 *file_name)
{
	Word32 size;
	FILE *f_ptr;
#ifdef DRAM
    accDRAM(FRAME_COUNT, 2);
#endif
	if ((f_ptr = fopen(file_name, "rb"))==NULL)
    {
        fprintf(stderr, "Error opening input file  %s !!\n", file_name);
        exit(0);
    }	
	fseek(f_ptr, 0, SEEK_END );
	size = ftell(f_ptr);
	fclose(f_ptr);
#ifdef DRAM
    accDRAM(FRAME_COUNT, -2);
#endif
	return(size/sizeof(Word16));
}
