/*
 --------------------------------------------------------------------------
 ITU-T G.720.1 Source Code
 Software Release 1.1a (2010-10)
 (C) 2009 Huawei Technologies Co., LTD. 
 --------------------------------------------------------------------------
*/

#include <math.h>
#include <stdio.h>
#include <memory.h>
#include <stdlib.h>
#include "wmops.h"
#ifndef GVAD
#include "stl_fx.h"
#include "vad_fx.h"
#include "smd_fx.h"
#include "parameters_fx.h"
#include "gsad_math_adv_fx.h"
#include "constdef_fx.h"
#include "Rom_fx.h"

extern int fluxId;
extern int hpId;
extern int smd_coreId;
extern int restId;


/**********************************************************************
* FUNCTION      :  smd_init_fx ()                                     *
*---------------------------------------------------------------------*
* PURPOSE       :  allocates and initializes smd state memory         *
*                                                                     *
*---------------------------------------------------------------------*
* INPUT ARGUMENTS  :                                                  *
*                 None                                                *
*---------------------------------------------------------------------*
* OUTPUT ARGUMENTS :                                                  *
*   (smdState *)  st     : initialized smd state                      *
*---------------------------------------------------------------------*
* INPUT/OUTPUT ARGUMENTS :                                            *
*                 None                                                *
*---------------------------------------------------------------------*
* RETURN ARGUMENTS :                                                  *
*   (Word32)             : non-zero with error, zero for ok           *
**********************************************************************/
Word16 smd_init_fx(smdState_fx **st_fx, Word16 nb)
{
   smdState_fx *s_fx; 
   if (st_fx == (smdState_fx **) NULL)
   {
	   return -1;
   }
   
   *st_fx = NULL;
   
   /* allocate memory */
   if ((s_fx = (smdState_fx *) malloc(sizeof(smdState_fx))) == NULL)
   {
      return -1;
   }
   smd_reset_fx(s_fx, nb);
   *st_fx = s_fx;
   return 0;
}	


/**********************************************************************
* FUNCTION      :  smd_reset_fx ()                                    *
*---------------------------------------------------------------------*
* PURPOSE       :  initializes smd state memory                       *
*                                                                     *
*---------------------------------------------------------------------*
* INPUT ARGUMENTS  :                                                  *
*                 None                                                *
*---------------------------------------------------------------------*
* OUTPUT ARGUMENTS :                                                  *
*                 None                                                *
*---------------------------------------------------------------------*
* INPUT/OUTPUT ARGUMENTS :                                            *
*   (smdState *)  st     : smd state                                  *
*---------------------------------------------------------------------*
* RETURN ARGUMENTS :                                                  *
*   (Word32)             : non-zero with error, zero for ok           *
**********************************************************************/
Word16 smd_reset_fx(smdState_fx *st_fx, Word16 nb)
{
	Word16 i;
#ifdef DRAM
    accDRAM(FRAME_COUNT, 1);
#endif

    if (st_fx==(smdState_fx *) NULL)
    {
		return -1;
    }

#ifdef WMOPS_FX
	move32();move32();
	move32();
	move16();move16();
	move16();move16();
	move16();move16();
	move16();move16();
	move16();move16();
	move16();move16();
	move16();move16();
	move16();move16();
	move16();move16();
#endif
	st_fx->fft_exp_min_ptr = 0;

    for (i=0; i<FLUX_F2_fx; i++) 
	{
#ifdef WMOPS_FX
    move16();
	move16();move16();
#endif	
		st_fx->SigFpwold_fx[i] = 0;
		st_fx->SigFpwold2_fx[i] = 0;
		st_fx->SigFpwold3_fx[i] = 0;
	}
	st_fx->Prev_fft_exp_fx[0] = 0;
	st_fx->Prev_fft_exp_fx[1] = 0;
	st_fx->Prev_fft_exp_fx[2] = 0;

	for (i=0; i<FLUXBUFSIZ_fx; i++) 
	{
#ifdef WMOPS_FX
    move16();
#endif
		st_fx->flux_buf_fx[i] = 0;
	}

	for (i=0; i<SMD_HIST_BUF_SIZE_fx; i++) 
	{
#ifdef WMOPS_FX
    move16();
#endif
		st_fx->var_flux_buf_fx[i] = 0;
	}
	st_fx->flux_vad_count_fx = 0;
	st_fx->flux_step_fx = 0;
	st_fx->mov_hp2_fx = 0;
    st_fx->mov_hp3_fx = 0;
	st_fx->count_first_snr_vad_fx = 0;
	st_fx->L_frame_cnt_fx = 0;
	st_fx->last_smd_raw_fx = 1;
	st_fx->last_smd_out_fx = 1;
	st_fx->diff_hist_sumt_fx = 0;

	for (i=0; i<SNR_SUMT_BUFF_SIZE_fx; i++) 
	{
#ifdef WMOPS_FX
    move16();
#endif
		st_fx->mssnr_buff_fx[i] = 0;
	}

	for (i=0; i<16; i++) 
	{
#ifdef WMOPS_FX
    move16();
#endif
		st_fx->mssnr_hist_decision_fx[i] = 0;
	}
	st_fx->mssnr_high_count_fx = 0;
	st_fx->mssnr_high_seg_count_fx = 0;
	st_fx->prev_high_level_seg_detect_fx = 0;
	st_fx->high_level_falling_edge_fx = 0;
	st_fx->falling_edge_cnt_fx = 0;
	st_fx->speech_reset_cnt_fx = -1;
	st_fx->N_high_200_fx = 0;
	st_fx->N_low_200_fx = 0;
#ifdef DRAM
    accDRAM(FRAME_COUNT, -1);
#endif
    return (0);
}


/**********************************************************************
* FUNCTION      :  smd_exit ()                                        *
*---------------------------------------------------------------------*
* PURPOSE       :  frees smd state memory                             *
*                                                                     *
*---------------------------------------------------------------------*
* INPUT ARGUMENTS  :                                                  *
*   (smdState *)  st     : smd state                                  *
*---------------------------------------------------------------------*
* OUTPUT ARGUMENTS :                                                  *
*                 None                                                *
*---------------------------------------------------------------------*
* INPUT/OUTPUT ARGUMENTS :                                            *
*                 None                                                *
*---------------------------------------------------------------------*
* RETURN ARGUMENTS :                                                  *
*                 None                                                *
**********************************************************************/
void smd_exit_fx(void **st_fx)
{
   if (st_fx==NULL || *st_fx==NULL)
   {
      return;
   }

   /* deallocate memory */
   free(*st_fx);
   *st_fx = NULL;
   return;
}


Word16 smd_core_fx(smdState_fx *st_fx, 
				   Word16 mssnr_fx, 
				   Word16 snr_vad, 
				   Word16 vad_flg, 
				   Word16 op, 
				   Word16 nb)
{
	Word16 i;
	Word16 smd_raw_fx;
	Word16 smd_out_fx;
	Word16 range_200_fx;
	Word16 T_sumt_hist_fx;
	Word16 T_200_var_flux_fx;
	Word16 diff_offset_fx;
	Word16 a_fx, b_fx;
	Word16 mssnr_hist_size_fx, hist_size_fx;
	Word16 end_of_buffer_decision_fx;
	Word16 biased_diff_fx;
	Word16 low_level_seg_detect_fx;	
	Word16 high_level_seg_detect_fx;
	Word16 mssnr_low_level_cnt_fx;
	Word16 T_search_range_fx;
	Word16 temp;
#ifdef DRAM
    accDRAM(FRAME_COUNT, 18);
#endif
#ifdef WMOPS_FX
    move16();
#endif
	mssnr_hist_size_fx = SNR_SUMT_HIST_SIZE_ACTUAL;

	IF (L_sub(L_deposit_h(mssnr_fx), st_fx->mssnr_max_fx)> 0) 
	{
		st_fx->mssnr_max_fx = L_deposit_h(mssnr_fx);
	}
	ELSE 
	{
		st_fx->mssnr_max_fx = L_mls(st_fx->mssnr_max_fx, 0x7FFC);    
	}
#ifdef WMOPS_FX
    move16();
#endif

	IF (nb) 
	{
		SWITCH (op) 
		{
			case 0:  //balance operating point
				T_sumt_hist_fx = round_fx(L_mls(st_fx->mssnr_max_fx, C_mssnr_BL_8K_fx));
				diff_offset_fx = K_offset_BL_8K_fx;
				a_fx = a_BL_8K_fx;
				b_fx = b_BL_8K_fx;
				break;
			case 1:  //quality operatin point
				T_sumt_hist_fx = round_fx(L_mls(st_fx->mssnr_max_fx, C_mssnr_QT_8K_fx));
				diff_offset_fx = K_offset_QT_8K_fx;
				a_fx = a_QT_8K_fx;
				b_fx = b_QT_8K_fx;
				break;
			case 2: // bandwidth saving
				T_sumt_hist_fx = round_fx(L_mls(st_fx->mssnr_max_fx, C_mssnr_BW_8K_fx));
				diff_offset_fx = K_offset_BW_8K_fx;
				a_fx = a_BW_8K_fx;
				b_fx = b_BW_8K_fx;
				break;
			default: 
				printf("operating point is %hd. Should be 0, 1, or 2 only\n",op);
				exit(0);
		}
	}
	ELSE 
	{
		SWITCH (op) 
		{
			case 0:  //balance operating point
				T_sumt_hist_fx = round_fx(L_mls(st_fx->mssnr_max_fx, C_mssnr_BL_16K_fx));
				diff_offset_fx = K_offset_BL_16K_fx;
				a_fx = a_BL_16K_fx;
				b_fx = b_BL_16K_fx;
				break;
			case 1:  //quality operatin point
				T_sumt_hist_fx = round_fx(L_mls(st_fx->mssnr_max_fx, C_mssnr_QT_16K_fx));
				diff_offset_fx = K_offset_QT_16K_fx;
				a_fx = a_QT_16K_fx;
				b_fx = b_QT_16K_fx;
				break;
			case 2: // bandwidth saving
				T_sumt_hist_fx = round_fx(L_mls(st_fx->mssnr_max_fx, C_mssnr_BW_16K_fx));
				diff_offset_fx = K_offset_BW_16K_fx;
				a_fx = a_BW_16K_fx;
				b_fx = b_BW_16K_fx;
				break;
			default: 
				printf("operating point is %hd. Should be 0, 1, or 2 only\n",op);
				exit(0);
		}
	}
	
	FOR (i=SNR_SUMT_BUFF_SIZE_fx-1; i>0; i--) 
	{
#ifdef WMOPS_FX
    move16();
#endif
		st_fx->mssnr_buff_fx[i] = st_fx->mssnr_buff_fx[sub(i, 1)];
	}
#ifdef WMOPS_FX
    move16();
#endif
	st_fx->mssnr_buff_fx[0] = mssnr_fx;

	IF (sub(mssnr_fx, 256) > 0) 
	{     
		st_fx->mssnr_high_seg_count_fx = add(st_fx->mssnr_high_seg_count_fx, 1);
	}
	else 
	{
		st_fx->mssnr_high_seg_count_fx = 0;
	}

	IF (sub(st_fx->mssnr_high_seg_count_fx, 500)>0) 
	{
		high_level_seg_detect_fx = 1;
	}
	else 
	{
		high_level_seg_detect_fx = 0;
	}

#ifdef WMOPS_FX
    test();
#endif
	IF (high_level_seg_detect_fx == 0 && st_fx->prev_high_level_seg_detect_fx) 
	{
		st_fx->high_level_falling_edge_fx = 1;
	}
#ifdef WMOPS_FX
    move16();
    move16();
#endif
	st_fx->prev_high_level_seg_detect_fx = high_level_seg_detect_fx;		
	
	low_level_seg_detect_fx = 0;		
	IF (st_fx->high_level_falling_edge_fx) 
	{
#ifdef WMOPS_FX
    move16();
#endif
		mssnr_low_level_cnt_fx = 0;
		FOR (i=0; i<SNR_SUMT_BUFF_SIZE_fx; i++) 
		{
			if (sub(st_fx->mssnr_buff_fx[i], 480)<0) 
			{    
				mssnr_low_level_cnt_fx = add(mssnr_low_level_cnt_fx, 1);
			}
		}
		if (sub(mssnr_low_level_cnt_fx, 40)>0) 
		{
			low_level_seg_detect_fx = 1;
		}
	}

#ifdef WMOPS_FX
    test();
#endif
	IF (low_level_seg_detect_fx && (sub(st_fx->speech_reset_cnt_fx, -1)==0)) 
	{
		st_fx->speech_reset_cnt_fx = 0;
	}
#ifdef WMOPS_FX
    test();
#endif
	IF ((st_fx->speech_reset_cnt_fx>=0) && (sub(st_fx->speech_reset_cnt_fx, 400)<0)) 
	{
		st_fx->speech_reset_cnt_fx = add(st_fx->speech_reset_cnt_fx, 1);
	}
	ELSE {
		st_fx->speech_reset_cnt_fx = -1;
	}
		
	if (st_fx->high_level_falling_edge_fx) 
	{
		st_fx->falling_edge_cnt_fx = add(st_fx->falling_edge_cnt_fx, 1);
	}
	if (sub(st_fx->falling_edge_cnt_fx, 150)==0) 
	{
		st_fx->high_level_falling_edge_fx = 0;
		st_fx->falling_edge_cnt_fx = 0;
	}

	IF (L_sub(st_fx->L_frame_cnt_fx, Start_diff_hist)>0) 
	{
#ifdef WMOPS_FX
    logic32();
	move16();
#endif
		if ((st_fx->mssnr_hist_decision_fx[15] & 0x80000000)!=0) 
		{
			end_of_buffer_decision_fx = 1;
		}
		else 
		{
			end_of_buffer_decision_fx = 0;
		}

		FOR (i=15; i>0; i--) 
		{
			IF ((st_fx->mssnr_hist_decision_fx[i-1] & 0x80000000)!=0) 
			{
#ifdef WMOPS_FX
    move32();
	logic32();
#endif
				st_fx->mssnr_hist_decision_fx[i] = (st_fx->mssnr_hist_decision_fx[i]<<1) | 0x00000001;
			}
			ELSE 
			{				
#ifdef WMOPS_FX
    move32();
#endif
				st_fx->mssnr_hist_decision_fx[i] = (st_fx->mssnr_hist_decision_fx[i]<<1);
			}
		}	
#ifdef WMOPS_FX
    move32();
#endif
		st_fx->mssnr_hist_decision_fx[0] = (st_fx->mssnr_hist_decision_fx[0]<<1);

		IF (sub(mssnr_fx, T_sumt_hist_fx)>0) 
		{
#ifdef WMOPS_FX
    logic32();
#endif
			st_fx->mssnr_hist_decision_fx[0] = st_fx->mssnr_hist_decision_fx[0] | 0x00000001;
			st_fx->mssnr_high_count_fx = add(st_fx->mssnr_high_count_fx, 1);
		}

		if (end_of_buffer_decision_fx>0) 
		{
			st_fx->mssnr_high_count_fx = sub(st_fx->mssnr_high_count_fx, 1);
		}

		IF (L_sub(st_fx->L_frame_cnt_fx, L_deposit_l(add(mssnr_hist_size_fx, Start_diff_hist)))<0) 
		{
			hist_size_fx = extract_l(L_sub(st_fx->L_frame_cnt_fx, Start_diff_hist));
		}
		else 
		{
#ifdef WMOPS_FX
    move16();
#endif
			hist_size_fx = mssnr_hist_size_fx;
		}

		IF (sub(st_fx->mssnr_high_count_fx, hist_size_fx)>=0) 
		{
#ifdef WMOPS_FX
    move16();
#endif
			temp = 0x4000;    
		}
		ELSE 
		{
			IF (st_fx->mssnr_high_count_fx<=0) 
			{
#ifdef WMOPS_FX
    move16();
#endif
				temp = 0;
			}
			ELSE 
			{
				temp = sub(div_s(st_fx->mssnr_high_count_fx, hist_size_fx), 0x4000);
			}
		}
		biased_diff_fx = add(temp, diff_offset_fx);
		st_fx->diff_hist_sumt_fx = round_fx(L_mac(L_mult(st_fx->diff_hist_sumt_fx, Stable_alpha_fx), One_minus_Stable_alpha_fx, biased_diff_fx));		
	}
	else 
	{       
#ifdef WMOPS_FX
    move16();
#endif	
		st_fx->diff_hist_sumt_fx = diff_offset_fx;
	}
	
	if (sub(st_fx->diff_hist_sumt_fx, X_limit_fx)>0)
	{
#ifdef WMOPS_FX
    move16();
#endif		
		st_fx->diff_hist_sumt_fx = X_limit_fx;
	}
		
	if (add(st_fx->diff_hist_sumt_fx, X_limit_fx)<0)
	{
#ifdef WMOPS_FX
    move16();
#endif
	    st_fx->diff_hist_sumt_fx = negate(X_limit_fx);
	}
		
	if (sub(st_fx->speech_reset_cnt_fx, 1)==0) 
	{
		st_fx->count_first_snr_vad_fx = 0;
	}
	if (st_fx->speech_reset_cnt_fx>0) 
	{
		st_fx->diff_hist_sumt_fx = diff_offset_fx;
	}
		
	T_200_var_flux_fx = add(mult_r(a_fx, st_fx->diff_hist_sumt_fx), b_fx);
#ifdef WMOPS_FX
    move16();
#endif	
	T_search_range_fx = T_SEARCH_RANGE;

	IF ((snr_vad ||(vad_flg && (st_fx->count_first_snr_vad_fx == 0))) && (L_sub(st_fx->L_frame_cnt_fx, Start_diff_hist) > 0)) 
	{ 
		st_fx->count_first_snr_vad_fx = add(st_fx->count_first_snr_vad_fx, 1);
		if (sub(st_fx->count_first_snr_vad_fx, T_SEARCH_RANGE) > 0) 
		{
#ifdef WMOPS_FX
    move16();
#endif
			st_fx->count_first_snr_vad_fx = T_SEARCH_RANGE;
		}

		if (sub(st_fx->count_first_snr_vad_fx, 75)<0) 
		{
			st_fx->N_high_200_fx = 75;
		}
		else 
		{
			st_fx->N_high_200_fx = 0;
		}

		st_fx->N_low_200_fx = 0;
		if (sub(st_fx->count_first_snr_vad_fx, T_search_range_fx)<0) 
		{
			range_200_fx = st_fx->count_first_snr_vad_fx;
		}
		else 
		{
			range_200_fx = T_search_range_fx;
		}

		FOR (i=0; i<range_200_fx; i++) 
		{
			if (sub(st_fx->var_flux_buf_fx[i], T_200_var_flux_fx)>0) 
			{
				st_fx->N_high_200_fx = add(st_fx->N_high_200_fx, 1);
			}
			if (sub(st_fx->var_flux_buf_fx[i] , T_200_var_flux_fx)<0) 
			{
				st_fx->N_low_200_fx = add(st_fx->N_low_200_fx, 1);
			}
		}
	}   

	if (sub(st_fx->N_high_200_fx, st_fx->N_low_200_fx)>=0) 
	{
		smd_raw_fx = 1;
	}
	else 
	{
		smd_raw_fx = 0;
	}

#ifdef WMOPS_FX
    test();
#endif
	IF (nb) 
	{
		IF ((L_sub(st_fx->mov_hp2_fx, 0x1B851EB8)>0) || (L_sub(st_fx->mov_hp3_fx, 0x3851EB85)>0)) 
		{
#ifdef WMOPS_FX
    move16();
#endif
			smd_raw_fx = 0;
		}
	}
	ELSE 
	{
		IF ((L_sub(st_fx->mov_hp2_fx, 0x2147AE14)>0) || (L_sub(st_fx->mov_hp3_fx, 0x4199999A)>0)) 
		{
#ifdef WMOPS_FX
    move16();
#endif
			smd_raw_fx = 0;
		}
	}

	if (sub(st_fx->last_smd_raw_fx, smd_raw_fx)==0) 
	{
		smd_out_fx = smd_raw_fx;
	}
	else 
	{
		smd_out_fx = st_fx->last_smd_out_fx;
	}
#ifdef WMOPS_FX
    move16();
	move16();
#endif
	st_fx->last_smd_raw_fx = smd_raw_fx;
	st_fx->last_smd_out_fx = smd_out_fx;
#ifdef DRAM
    accDRAM(FRAME_COUNT, -18);
#endif	
	return(smd_out_fx);
}


/**********************************************************************
* FUNCTION      :  smd_fx ()                                          *
*---------------------------------------------------------------------*
* PURPOSE       :  outputs the final smd result                       *
*                                                                     *
*---------------------------------------------------------------------*
* INPUT ARGUMENTS  :                                                  *
*   (Word16)     vad_flg      : voice activity indicator              *
*   (Word16)     nb           : sampling rate flag                    *
*   (Word16)     op           : operating point flag                  *
*   (Float32 *)  SigFpw       : pointer to the spectrum power         *
*                               density of the input frame            *
*---------------------------------------------------------------------*
* OUTPUT ARGUMENTS :                                                  *
*                None                                                 *
*---------------------------------------------------------------------*
* INPUT/OUTPUT ARGUMENTS :                                            *
*   (smdState *) st           : the smd state                         *
*---------------------------------------------------------------------*
* RETURN ARGUMENTS :                                                  *
*   (Word16)     output       : signal class indicator (speech/music) *
**********************************************************************/
Word16 smd_fx(smdState_fx *st_fx, 
			  Word16 *SigFpw_fx, 
			  Word16 fft_exp_fx, 
			  Word16 vad_flg, 
			  Word16 mssnr_fx, 
			  Word16 op, 
			  Word16 nb)
{
	Word16 output_fx;
	Word16 snr_vad_fx;
#ifdef DRAM
    accDRAM(FRAME_COUNT, 2);
#endif
#ifdef WMOPS_FX
    move16();
#endif	
	if (mssnr_fx>1600) 
	{
		snr_vad_fx = 1;
	}
	else 
	{
		snr_vad_fx = 0;
	}
	
    st_fx->L_frame_cnt_fx = L_add(st_fx->L_frame_cnt_fx, 1);
#ifdef DETAIL
    setCounter(fluxId); 
#endif
	parametersSMD_flux_fx(st_fx, snr_vad_fx, fft_exp_fx, SigFpw_fx, nb); 
#ifdef DETAIL
    setCounter(hpId); 
#endif
	parametersSMD_hp_fx(st_fx, SigFpw_fx, nb, snr_vad_fx);
#ifdef DETAIL
    setCounter(smd_coreId); 
#endif
    output_fx = smd_core_fx(st_fx, mssnr_fx, snr_vad_fx, vad_flg, op, nb);
#ifdef DETAIL
    setCounter(restId); 
#endif
#ifdef DRAM
    accDRAM(FRAME_COUNT, -2);
#endif
	return (output_fx);
}


Word16 sid_fx(vadState_fx *st_fx) 
{   
	Word16 sid_flag;
    Word16 level_tmp;
#ifdef DRAM
    accDRAM(FRAME_COUNT, 2);
#endif
	
	level_tmp = log10_Q15(st_fx->rms_Input_fx);                                                        BRANCH(1);ADD(1);INDIRECT(1);MULT(2);TRANS(1);               
#ifdef WMOPS_FX
    move16();
#endif
	IF (sub(level_tmp, (Word16)0xD333) < 0)
	{
		sid_flag = 0;                                        
	}
	ELSE
	{
		sid_flag = 1;                                        
	}
#ifdef DRAM
    accDRAM(FRAME_COUNT, -2);
#endif	
	return (sid_flag);
}


Word16 sad_output(Word16 vad_flg, Word16 smd_flag, Word16 sid_flag)
{   
	Word16 sad_flag;
#ifdef DRAM
    accDRAM(FRAME_COUNT, 1);
#endif

	IF (vad_flg)
	{
		sad_flag = (add(shl(vad_flg, 1) , smd_flag));
	}
	ELSE
	{
		sad_flag = (add(shl(vad_flg, 1) , sid_flag));
	}
#ifdef DRAM
    accDRAM(FRAME_COUNT, -1);
#endif
	return (sad_flag);
}

#endif
