/*
 --------------------------------------------------------------------------
 ITU-T G.720.1 Source Code
 Software Release 1.1a (2010-10)
 (C) 2000 VoiceAge Corporation, 2009 Huawei Technologies Co., Ltd.
 --------------------------------------------------------------------------
*/

/*
 * wmops.h
 *
 * Cycle counting library routines for WMOPS
 * (weighted millions of operations per second)
 * estimation.
 *
 */


#ifndef WMOPS_H
#define WMOPS_H

#include "typedef_fx.h"

#ifdef DRAM
/* -- Dynamic RAM estimation tool - Huawei Technologies Co., Ltd.  */

#define FRAME_RESET 0
#define FRAME_COUNT 1
#define DRAM_REPORT 2
int accDRAM(int flag,int c);
/**************************/

/*
 * Enable WMOPS counters by -DWMOPS or defining WMOPS in lib_com\switch.c or
 * by manually setting ENABLE_WMOPS to 1
 */
#endif /* HW_DRAM */

#ifdef  WMOPS
#define ENABLE_WMOPS 1
#else
#define ENABLE_WMOPS 0
#endif

/*
 * Enable Call Tree by -DTREE or
 * by manually setting ENABLE_TREE to 1
 *
 */

#ifdef  TREE
#define ENABLE_TREE 1
#else
#define ENABLE_TREE 0
#endif

/* 
 * If WMOPS and/or TREE enabled, then define interfaces accordingly,
 * else define interfaces as NULLs
 *
 */

#if (ENABLE_WMOPS|ENABLE_TREE)

void reset_wmops (void);
void push_wmops (const char *label);
void pop_wmops (void);
void update_wmops (void);
void print_wmops (void);

#else

#define reset_wmops()
#define push_wmops(x)
#define pop_wmops()
#define update_wmops()
#define print_wmops()

#endif


#if ENABLE_WMOPS

#define NUM_INST 16   // Total number of instruction types (in enum below)

extern double ops_cnt;
extern double prom_cnt;
extern double inst_cnt [NUM_INST];
extern int    ignore_depth;
enum instructions {_ADD,_ABS, _MULT, _MAC, _MOVE, _STORE, _LOGIC, _SHIFT, _BRANCH, _DIV, _SQRT, _TRANS, _FUNC, _LOOP, _INDIRECT, _PTR_INIT};

#define _ADD_C      1
#define _ABS_C      1
#define _MULT_C     1
#define _MAC_C      1
#define _MOVE_C     1
#define _STORE_C    1
#define _LOGIC_C    1
#define _SHIFT_C    1
#define _BRANCH_C   2
#define _DIV_C     18
#define _SQRT_C    25
#define _TRANS_C   25
#define _FUNC_C     2  // need to add number of arguments
#define _LOOP_C     1
#define _INDIRECT_C 2
#define _PTR_INIT_C 1

#define _ADD_P      1
#define _ABS_P      1
#define _MULT_P     1
#define _MAC_P      1
#define _MOVE_P     1
#define _STORE_P    0
#define _LOGIC_P    1
#define _SHIFT_P    1
#define _BRANCH_P   2
#define _DIV_P      2
#define _SQRT_P     2
#define _TRANS_P    2
#define _FUNC_P     2  // need to add number of arguments
#define _LOOP_P     1
#define _INDIRECT_P 2
#define _PTR_INIT_P 1


#define IGNORE()    { if (ignore_depth == 0) { ignore_depth = 1;}}

#define ADD(x)      { if (ignore_depth == 0) { ops_cnt+=(_ADD_C*(x));      inst_cnt[_ADD]+=(x);      { static int pcnt; if (!pcnt) { pcnt=1; prom_cnt+=(_ADD_P*(x));      }}}}
#define ABS(x)      { if (ignore_depth == 0) { ops_cnt+=(_ABS_C*(x));      inst_cnt[_ABS]+=(x);      { static int pcnt; if (!pcnt) { pcnt=1; prom_cnt+=(_ABS_P*(x));      }}}}
#define MULT(x)     { if (ignore_depth == 0) { ops_cnt+=(_MULT_C*(x));     inst_cnt[_MULT]+=(x);     { static int pcnt; if (!pcnt) { pcnt=1; prom_cnt+=(_MULT_P*(x));     }}}}
#define MAC(x)      { if (ignore_depth == 0) { ops_cnt+=(_MAC_C*(x));      inst_cnt[_MAC]+=(x);      { static int pcnt; if (!pcnt) { pcnt=1; prom_cnt+=(_MAC_P*(x));      }}}}
#define MOVE(x)     { if (ignore_depth == 0) { ops_cnt+=(_MOVE_C*(x));     inst_cnt[_MOVE]+=(x);     { static int pcnt; if (!pcnt) { pcnt=1; prom_cnt+=(_MOVE_P*(x));     }}}}
#define STORE(x)    { if (ignore_depth == 0) { ops_cnt+=(_STORE_C*(x));    inst_cnt[_STORE]+=(x);    { static int pcnt; if (!pcnt) { pcnt=1; prom_cnt+=(_STORE_P*(x));    }}}}
#define LOGIC(x)    { if (ignore_depth == 0) { ops_cnt+=(_LOGIC_C*(x));    inst_cnt[_LOGIC]+=(x);    { static int pcnt; if (!pcnt) { pcnt=1; prom_cnt+=(_LOGIC_P*(x));    }}}}
#define SHIFT(x)    { if (ignore_depth == 0) { ops_cnt+=(_SHIFT_C*(x));    inst_cnt[_SHIFT]+=(x);    { static int pcnt; if (!pcnt) { pcnt=1; prom_cnt+=(_SHIFT_P*(x));    }}}}
#define BRANCH(x)   { if (ignore_depth == 0) { ops_cnt+=(_BRANCH_C*(x));   inst_cnt[_BRANCH]+=(x);   { static int pcnt; if (!pcnt) { pcnt=1; prom_cnt+=(_BRANCH_P*(x));   }}}}
#define DIV(x)      { if (ignore_depth == 0) { ops_cnt+=(_DIV_C*(x));      inst_cnt[_DIV]+=(x);      { static int pcnt; if (!pcnt) { pcnt=1; prom_cnt+=(_DIV_P*(x));      }}}}
#define SQRT(x)     { if (ignore_depth == 0) { ops_cnt+=(_SQRT_C*(x));     inst_cnt[_SQRT]+=(x);     { static int pcnt; if (!pcnt) { pcnt=1; prom_cnt+=(_SQRT_P*(x));     }}}}
#define TRANS(x)    { if (ignore_depth == 0) { ops_cnt+=(_TRANS_C*(x));    inst_cnt[_TRANS]+=(x);    { static int pcnt; if (!pcnt) { pcnt=1; prom_cnt+=(_TRANS_P*(x));    }}}}
#define LOOP(x)     { if (ignore_depth == 0) { ops_cnt+=(_LOOP_C*(x));     inst_cnt[_LOOP]+=(x);     { static int pcnt; if (!pcnt) { pcnt=1; prom_cnt+=(_LOOP_P*(x));     }}}}
#define INDIRECT(x) { if (ignore_depth == 0) { ops_cnt+=(_INDIRECT_C*(x)); inst_cnt[_INDIRECT]+=(x); { static int pcnt; if (!pcnt) { pcnt=1; prom_cnt+=(_INDIRECT_P*(x)); }}}}
#define PTR_INIT(x) { if (ignore_depth == 0) { ops_cnt+=(_PTR_INIT_C*(x)); inst_cnt[_PTR_INIT]+=(x); { static int pcnt; if (!pcnt) { pcnt=1; prom_cnt+=(_PTR_INIT_P*(x)); }}}}
                                             
#define FUNC(x)     { if (ignore_depth == 0) { ops_cnt+=(_FUNC_C+_MOVE_C*(x)); inst_cnt[_FUNC]++; inst_cnt[_MOVE]+=(x); { static int pcnt; if (!pcnt) { pcnt=1; prom_cnt+=(_FUNC_P+_MOVE_P*(x)); }}}}

#define DADD(x)     { if (ignore_depth == 0) { ops_cnt+=(2*_ADD_C*(x));    inst_cnt[_ADD]+=(x);      { static int pcnt; if (!pcnt) { pcnt=1; prom_cnt+=(_ADD_P*(x));      }}}}
#define DMULT(x)    { if (ignore_depth == 0) { ops_cnt+=(2*_MULT_C*(x));   inst_cnt[_MULT]+=(x);     { static int pcnt; if (!pcnt) { pcnt=1; prom_cnt+=(_MULT_P*(x));     }}}}
#define DMAC(x)     { if (ignore_depth == 0) { ops_cnt+=(2*_MAC_C*(x));    inst_cnt[_MAC]+=(x);      { static int pcnt; if (!pcnt) { pcnt=1; prom_cnt+=(_MAC_P*(x));      }}}}
#define DMOVE(x)    { if (ignore_depth == 0) { ops_cnt+=(2*_MOVE_C*(x));   inst_cnt[_MOVE]+=(x);     { static int pcnt; if (!pcnt) { pcnt=1; prom_cnt+=(_MOVE_P*(x));     }}}}
#define DSTORE(x)   { if (ignore_depth == 0) { ops_cnt+=(2*_STORE_C*(x));  inst_cnt[_STORE]+=(x);    { static int pcnt; if (!pcnt) { pcnt=1; prom_cnt+=(_STORE_P*(x));    }}}}
#define DLOGIC(x)   { if (ignore_depth == 0) { ops_cnt+=(2*_LOGIC_C*(x));  inst_cnt[_LOGIC]+=(x);    { static int pcnt; if (!pcnt) { pcnt=1; prom_cnt+=(_LOGIC_P*(x));    }}}}
#define DSHIFT(x)   { if (ignore_depth == 0) { ops_cnt+=(2*_SHIFT_C*(x));  inst_cnt[_SHIFT]+=(x);    { static int pcnt; if (!pcnt) { pcnt=1; prom_cnt+=(_SHIFT_P*(x));    }}}}
#define DDIV(x)     { if (ignore_depth == 0) { ops_cnt+=(2*_DIV_C*(x));    inst_cnt[_DIV]+=(x);      { static int pcnt; if (!pcnt) { pcnt=1; prom_cnt+=(_DIV_P*(x));      }}}}
#define DSQRT(x)    { if (ignore_depth == 0) { ops_cnt+=(2*_SQRT_C*(x));   inst_cnt[_SQRT]+=(x);     { static int pcnt; if (!pcnt) { pcnt=1; prom_cnt+=(_SQRT_P*(x));     }}}}
#define DTRANS(x)   { if (ignore_depth == 0) { ops_cnt+=(2*_TRANS_C*(x));  inst_cnt[_TRANS]+=(x);    { static int pcnt; if (!pcnt) { pcnt=1; prom_cnt+=(_TRANS_P*(x));    }}}}


#else

#define IGNORE()

#define ADD(x)
#define ABS(x)
#define MULT(x)
#define MAC(x)
#define MOVE(x)
#define STORE(x)
#define LOGIC(x)
#define SHIFT(x)
#define BRANCH(x)
#define DIV(x)
#define SQRT(x)
#define TRANS(x)
#define FUNC(x)
#define LOOP(x)
#define INDIRECT(x)
#define PTR_INIT(x)

#define DADD(x)
#define DMULT(x)
#define DMAC(x)
#define DMOVE(x)
#define DSTORE(x)
#define DLOGIC(x)
#define DSHIFT(x)
#define DDIV(x)
#define DSQRT(x)
#define DTRANS(x)

#endif


#endif
